#!/bin/bash

### Parameters
# Information to put in the smart poster
#TITLE=$'\U0001f489 \u24ba\u24ca \u24b9\u24d8\u24d6\u24d8\u24e3\u24d0\u24db \u24b8\u24c4\u24cb\u24be\u24b9 \u24b8\u24d4\u24e1\u24e3\u24d8\u24d5\u24d8\u24d0\u24e3\u24d4 \U0001f489'
MAIN_TITLE=$'\U0001f489 \U0001f134\U0001f144 \U0001f133\U0001f138\U0001f136\U0001f138\U0001f143\U0001f130\U0001f13b \U0001f132\U0001f13e\U0001f145\U0001f138\U0001f133 \U0001f132\U0001f134\U0001f141\U0001f143\U0001f138\U0001f135\U0001f138\U0001f132\U0001f130\U0001f143\U0001f134 \U0001f489'
SUBTITLE=$'\U0001f145\U0001f130\U0001f132\U0001f132\U0001f138\U0001f13d\U0001f130\U0001f143\U0001f138\U0001f13e\U0001f13d \U0001f132\U0001f134\U0001f141\U0001f143\U0001f138\U0001f135\U0001f138\U0001f132\U0001f130\U0001f143\U0001f134'
BULLET=$'\u2023'
NAME="Testaaja, Matti Kari"
DATE_OF_BIRTH="1995-05-20"
UNIQUE_CERTIFICATE_IDENTIFIER="URN:UVCI:01:FI:41EPCEEXEDR3TZV5RW9XQIR15#2"
DOSES_IN_SERIES="2 / 2"
DATE_OF_VACCINATION="2021-03-05"
EU_FLAG="../images/EU_flag-360x270.png"
EU_FLAG_MIMETYPE="image/png"
QR_CODE="../images/QR_code-924x924.png"
QR_CODE_MIMETYPE="image/png"
URI="https://www.kanta.fi/documents/20143/120102/mallitodistus_eu_rokotustodistus.pdf/7abaee69-2ec2-7462-85b0-80ab073d9212?t=1623911512376"

# Output file
NDEF_FILE="smart_poster.ndef"

# Path to the utilities we need
NDEFTOOL="ndeftool"
TAGTOOL="python3 ../utils/nfcpy/examples/tagtool.py"
 


### NDEF generator
NAME_RECORD=$(echo -en "Surname(s) and forename(s):\n\t$BULLET $NAME")
DATE_OF_BIRTH_RECORD=$(echo -en "Date of birth:\n\t$BULLET $DATE_OF_BIRTH")
UNIQUE_CERTIFICATE_IDENTIFIER_RECORD=$(echo -en "Unique certificate identifier:\n\t$BULLET $UNIQUE_CERTIFICATE_IDENTIFIER")
DOSES_IN_SERIES_RECORD=$(echo -en "Number in a series of vaccinations/doses and the overall number of doses in the series:\n\t$BULLET $DOSES_IN_SERIES")
DATE_OF_VACCINATION_RECORD=$(echo -en "Date of vaccination, indicating the date of the latest dose received:\n\t$BULLET $DATE_OF_VACCINATION")
AGENT_TARGETED_RECORD=$(echo -en "Disease or agent targeted:\n\t$BULLET COVID-19")
VACCINE_RECORD=$(echo -en "Vaccine:\n\t$BULLET COVID-19 vaccines")
MEDICINAL_PRODUCT_RECORD=$(echo -en "Vaccine medicinal product:\n\t$BULLET Comirnaty")
AUTHORIZATION_HOLDER_RECORD=$(echo -en "Vaccine marketing authorisation holder:\n\t$BULLET BioNTech Manufacturing GmbH")
MEMBER_STATE_RECORD=$(echo -en "Member state of vaccination:\n\t$BULLET Finland")
CERTIFICATE_ISSUER_RECORD=$(echo -en "Certificate issuer:\n\t$BULLET The Social Insurance Institution of Finland\n\t$BULLET Kela/Fpa")

# Create the NDEF record
echo "Encapsulating the data into a smart poster NDEF"
$NDEFTOOL \
	smp "$MAIN_TITLE" \
	load --pack $EU_FLAG tn $EU_FLAG_MIMETYPE id '' \
	txt "$NAME_RECORD" \
	txt "$DATE_OF_BIRTH_RECORD" \
	txt "$UNIQUE_CERTIFICATE_IDENTIFIER_RECORD" \
	load --pack $QR_CODE tn $QR_CODE_MIMETYPE id ''\
	txt "$SUBTITLE" \
	txt "$AGENT_TARGETED_RECORD" \
	txt "$VACCINE_RECORD" \
	txt "$MEDICINAL_PRODUCT_RECORD" \
	txt "$AUTHORIZATION_HOLDER_RECORD" \
	txt "$DOSES_IN_SERIES_RECORD" \
	txt "$DATE_OF_VACCINATION_RECORD" \
	txt "$MEMBER_STATE_RECORD" \
	txt "$CERTIFICATE_ISSUER_RECORD" \
	uri "$URI" \
	save --force $NDEF_FILE

# Dump the size of the NDEF file, for information
NDEFSIZE=$(wc -c < $NDEF_FILE)
echo "NDEF size: $NDEFSIZE bytes"

# Read the tag
echo "Reading the tag"
CARD_INFO=$($TAGTOOL)

# Make sure the tag is writable
if [ ! "$(echo $CARD_INFO | grep -i 'NDEF Capabilities.*writeable *= *yes')" ]; then
  echo "Tag is not writeable!"
  exit -1
fi

# Make sure there's enough room on the tag for the NDEF record
TCAPA=$(echo $CARD_INFO | sed -r 's/^.*NDEF Capabilities.*capacity *= *([0-9]+) byte.*$/\1/')
echo "Tag's capacity: $TCAPA bytes"
if [ $TCAPA -lt $NDEFSIZE ]; then
  echo "NDEF size exceeds the tag's capacity!"
  exit -1
fi

# Ask the user if they really want to write the NDEF on the tag
read -p "Write NDEF onto the tag [Y/N]? " YN
if [ "$YN" != "y" ] && [ "$YN" != "Y" ]; then
  echo "Aborting"
  exit 0
fi

# Write the NDEF file onto the tag
echo "Writing NDEF onto the tag"
$TAGTOOL load $NDEF_FILE

echo "All done!"
